import 'dart:convert';
import 'dart:io';
import 'dart:isolate';

import 'package:enx_flutter_plugin/enx_flutter_plugin.dart';
import 'package:enx_uikit_flutter/models/ChatModel.dart';
import 'package:enx_uikit_flutter/models/active_talker.dart';
import 'package:enx_uikit_flutter/models/chat_list_model.dart';
import 'package:enx_uikit_flutter/models/file_share_model.dart';
import 'package:enx_uikit_flutter/models/floor_request_model.dart';
import 'package:enx_uikit_flutter/models/more_model.dart';
import 'package:enx_uikit_flutter/models/setting_model.dart';
import 'package:enx_uikit_flutter/models/setting_model_list.dart';
import 'package:enx_uikit_flutter/models/user_list_model.dart';
import 'package:enx_uikit_flutter/models/user_model.dart';
import 'package:enx_uikit_flutter/utilities/enx_setting.dart';
import 'package:enx_uikit_flutter/utilities/utils.dart';

import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:flutter/services.dart';
import 'package:flutter_foreground_task/flutter_foreground_task.dart';
import 'package:get/get.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';
import 'package:get/get_rx/src/rx_types/rx_types.dart';
import 'package:get/get_state_manager/src/simple/get_controllers.dart';
import 'package:get_storage/get_storage.dart';
import 'package:permission_handler/permission_handler.dart';


class EnxController extends GetxController {
  String token;
  var isAudioMuted = false.obs;
  var isVideoMuted = false.obs;
  var isModerator = false.obs;
  var isRecording = false.obs;
  var isHardMute = false.obs;
  var isShow = false.obs;
  var isChatIcon = false.obs;
  var isNotificationIcon = false.obs;
  var isShowLocalRemote = true.obs;
  var isHardMuteAudio = false.obs;
  var hardMuteVideo = false.obs;
  var isScreenShared = false.obs;
  var selectedDevice = 'EARPIECE'.toUpperCase().obs;
  var isGrid = true.obs;
  var isActive = false.obs;
  var isConfirmation=true.obs;
  BuildContext context;

  bool isMuted = false;
  bool isMuteVideo = false;
  var backCamera = false.obs;
  var appMode = 'Group'.toUpperCase().obs;
  var chatType = 'group'.obs;
  var isScreenSharedStarted = false.obs;

  var screenShareStreamId = 0.obs;
  var positionCom=0.0.obs;

  double get positionUpdate => positionCom.value;


  var isLongPress = false.obs;

  var loadingTxt = 'Please wait loading...'.obs;

  var _dataAvailable = false.obs;

  var activeTalkerList = <ActiveList>[].obs;

  var totalSubscriberInRoom = 0.0.obs;

  var progressValueInstream = 0.0;

  var streamId = 0.obs;

  var isShowAnnotation = false.obs;

  var participantFloorAction = 0.obs;

  var currentClientPosition = -1;

  UserModel? userModelSelf;

  MoreModel? moreModel;

  ChatModel? chatModel;

  FloorRequestModel? floorRequestModel;

  var userList = <UserListModel>[].obs;
  var userList1 = <UserListModel>[].obs;

  var participantsList = <UserModel>[].obs;
  var moderatorsList = <UserModel>[].obs;
  var moreList = <MoreModel>[].obs;
  var floorRequestList = <FloorRequestModel>[].obs;

  var deviceList = <dynamic>[].obs;
  var chatList = <ChatModel>[].obs;
  var privateChatList = <ChatModel>[].obs;

  var privateChatListData = <ChatListModel>[].obs;

  void Function(Map<dynamic, dynamic> map) disconnect;

  void Function(Map<dynamic, dynamic> map) connectError;
  void Function(Map<dynamic, dynamic> map) pageSlide;


  final ScrollController scrollController = ScrollController();

  var setting = EnxSetting.instance;


  late String roomID;

  late String clientID;
  late ReceivePort _receivePort;
  var fileShareModel;
  var settingList = <SettingModelList>[].obs;
var isTyping=false.obs;
  var isTypingStarted = false.obs;
  var typerName="".obs;



//webinar feature
  EnxController(this.token, this.context, this.disconnect, this.connectError,this.pageSlide);

  bool get dataAvailable => _dataAvailable.value;

  int get floorAction => participantFloorAction.value;

  String get getChatType => chatType.value;

  bool get isActiveTalker => isActive.value;

  bool get isAudioMute => isAudioMuted.value;

  bool get isBackCamera => backCamera.value;

  bool get isChatGroupIcon => isChatIcon.value;

  bool get isGridView => isGrid.value;

  bool get isHardMuteRoom => isHardMute.value;

  bool get isHardMuteRoomAudio => isHardMuteAudio.value;

  bool get isHardMuteVideo => hardMuteVideo.value;

  bool get isLongPressEnable => isLongPress.value;

  bool get isNotification => isNotificationIcon.value;

  bool get isRecord => isRecording.value;

  bool get isRole => isModerator.value;

  bool get isScreenShare => isScreenShared.value;

  bool get isScreenShareStarted => isScreenSharedStarted.value;

  bool get isShowAnnotationView => isShowAnnotation.value;

  bool get isShowHide => isShow.value;

  bool get isVideoMute => isVideoMuted.value;

  bool get isVisibleLocalRemote => isShowLocalRemote.value;

  bool get showConfirmation=>isConfirmation.value;

  String get loading => loadingTxt.value;

  String get mode => appMode.value;

  double get progressValue => totalSubscriberInRoom.value;

  int get screenShareStream => screenShareStreamId.value;

  //butthons
  String get selectedMedia => selectedDevice.value;

  int get streamIdForAnnotation => streamId.value;
  var isSelect = [false, false,false].obs;
var isAudioOnly=false.obs;
var isVideoSelected=false.obs;
var isSetting=false.obs;
bool get isSettingShow=>isSetting.value;
bool get audioOnly=>isAudioOnly.value;
  List<bool> get isSelected =>isSelect.value;

  Future<void> acceptFloors(String clientId) async {
    EnxRtc.grantFloor(clientId);
  }

  void addParticipantsList(RxList<UserListModel> userList) {
    for (var model in userList) {
      var chatModelList = ChatListModel(model.name!, model.clientId!, []);
      privateChatListData.add(chatModelList);

      if (model.role == 'participant') {
        var userModel = UserModel(model.clientId!, model.name!, model.role!,
            model.audioMuted!, model.videoMuted!, false);
        participantsList.add(userModel);
      } else {
        var userModel = UserModel(model.clientId!, model.name!, model.role!,
            model.audioMuted!, model.videoMuted!, false);
        moderatorsList.add(userModel);
      }
    }
    moderatorsList.refresh();
    participantsList.refresh();
  }

  Future<void> annotationShare() async {
    if (Platform.isAndroid) {
      if (isScreenShare) {
        EnxRtc.stopAnnotation();
        _stopForegroundTask();
      } else {
        _startForegroundTask();
        EnxRtc.startAnnotation(streamIdForAnnotation.toString());
      }
    } else if (Platform.isIOS) {
      if (isScreenShare) {
        EnxRtc.stopAnnotation();
      } else {
        EnxRtc.startAnnotation(streamId.toString());
      }
    }
  }

  Future<void> cancelFloors() async {
    EnxRtc.cancelFloor();
  }

  Future<void> dropCall(List<String> clientIds) async {
    await EnxRtc.dropUser(clientIds);

    // dispose();
  }

  Future<void> endCall(String screen) async {
    if (screen == 'participantScreen') {
      Get.back();
    }
    await EnxRtc.disconnect();

    // dispose();
  }

  Future<void> finishFloors() async {
    EnxRtc.finishFloor();
  }

  void groupChatView() {
    /*Get.to(() => ChatPage(chatList),
        transition: Transition.leftToRight,
        duration: const Duration(seconds: 1));*/
  }

  Future<void> initEnxRtc() async {

    _addEnxEventHandlers();
    Map<String, dynamic> map = {
      'minWidth': 320,
      'minHeight': 180,
      'maxWidth': 1280,
      'maxHeight': 720
    };

    Map<String, dynamic> localInfo = {
      'audio': true,
      'video': true,
      'data': true,
      'framerate': 30,
      'audioMuted': isSelected.first,
      'videoMuted': isSelected[1],
      'videoSize': map,
      'audioOnlyMode':EnxSetting.instance.isConfirmation?isSelected[2]:EnxSetting.instance.isAudioOnlyCalls,
      'isVideoResolution': true
    };

    Map<String, dynamic> playerConfig = {
      'bandwidth': true,
      'screenshot': true,
      'avatar': true,
      'pinned': true,
      'iconHeight': 25,
      'iconWidth': 25,
     // "iconColor": '#ffffff'
    };
    Map<String, dynamic> roomInfo = {
      'allow_reconnect': true,
      'number_of_attempts': 3,
      'timeout_interval': 45,
      'playerConfiguration': playerConfig,
      'forceTurn': false,
      'chat_only': false
    };

    // bool isPermissionGranted =
    // await handlePermissionsForCall(context);
    //if (isPermissionGranted) {
    await EnxRtc.joinRoom(token, localInfo, roomInfo, List.empty());
    // }
  }

  void moreListItem(String role) {
    if (role == 'moderator') {
      moreList.add(MoreModel.name(
          'Switch Layout',
          MenuOption.switchAT,
          'assets/image/gridview.png',
          'assets/image/gridview_on.png',
          false,
          true));
      moreList.add(MoreModel.name(
          'Mute Room',
          MenuOption.muteRoom,
          'assets/image/moderator_unmute.png',
          'assets/image/moderator_mute.png',
          false,
          true));
      moreList.add(MoreModel.name(
          'Recording On',
          MenuOption.recording,
          'assets/image/recording.png',
          'assets/image/recording_on.png',
          false,
          true));

      //moderatorsList.add(item);
    } else {
      moreList.add(MoreModel.name(
          'Switch Layout',
          MenuOption.switchAT,
          'assets/image/gridview.png',
          'assets/image/gridview_on.png',
          false,
          true));

      // participantsList.add(item);
    }
    moreList.add(MoreModel.name('Request List', MenuOption.requestFloor,
        'assets/image/notification_icon.png', 'assets/image/notification_icon_selected.png', false,false));
    // moreList.add(MoreModel.name('Start fsAnnotation', MenuOption.annotationShare,
    //     'assets/image/gridview.png', 'assets/image/gridview_on.png', false,false));
    // moreList.add(MoreModel.name('Start Annotationvc', MenuOption.annotationShare,
    //     'assets/image/gridview.png', 'assets/image/gridview_on.png', false,false));
  }

  void checkSettingEnable() async{
    if(GetStorage().hasData("setting")){
      var map= await GetStorage().read("setting");
      var map1=jsonDecode(map) as List;

      for(int i=0;i<map1.length;i++){
        var conferenceModelList = <SettingModel>[];

        var subList= map1[i]['settingList'] as List;
        for(int j=0;j<subList.length;j++){

          conferenceModelList.add(SettingModel(subList[j]['name'], subList[j]['isStatus']));
        }
        settingList.add(SettingModelList(map1[i]['name'], conferenceModelList));
        print("asdsd ${map1[i]['settingList']}");
      }

    }
  }
  void settingListItem()  async{
    settingList.clear();
    if(GetStorage().hasData("setting")){
      var map= await GetStorage().read("setting");
      final split = map.split(',');
     var map1=jsonDecode(map) as List;

      for(int i=0;i<map1.length;i++){
        var conferenceModelList = <SettingModel>[];

        var subList= map1[i]['settingList'] as List;
         for(int j=0;j<subList.length;j++){
           conferenceModelList.add(SettingModel(subList[j]['name'], subList[j]['isStatus']));
         }
          settingList.add(SettingModelList(map1[i]['name'], conferenceModelList));
          print("asdsd ${map1[i]['settingList']}");
        }

    }else {
      var conferenceModelList = <SettingModel>[];
      conferenceModelList
          .add(SettingModel("Ask To Confirm When Leaving Meeting", false));
      conferenceModelList.add(SettingModel("Auto Copy Invite Link", false));
      conferenceModelList.add(SettingModel("Show My Connected Time", false));
      settingList
          .add(SettingModelList("Conference Settings", conferenceModelList));

      var meetingSoundsList = <SettingModel>[];
      meetingSoundsList.add(SettingModel("New Chat Message", false));
      meetingSoundsList.add(SettingModel("Beep on Entry", false));
      meetingSoundsList.add(SettingModel("Beep on Exit", false));
      meetingSoundsList.add(SettingModel("Start Recording", false));
      meetingSoundsList.add(SettingModel("Stop Recording", false));
      meetingSoundsList.add(SettingModel("End of Meeting", false));
      settingList.add(SettingModelList("Meeting Sounds", meetingSoundsList));

      // settingList.toString()


      String jsonString = jsonEncode(settingList);
       print("GetStorage ${jsonString}");

      GetStorage().write("setting",jsonString);
    }

  }

  Future<void> muteRoom() async {
    if (isHardMuteRoom) {
      EnxRtc.hardUnMute();
    } else {
      EnxRtc.hardMute();
    }
    // EnxRtc.hardUnMute();
  }

  @override
  void onClose() {
    scrollController.dispose();
    super.onClose();
  }

  void onGrantDenyReleaseManagement(Map<dynamic, dynamic> map) {
    final result = map["result"];
    final msg = map["msg"];
    print("releasw${jsonEncode(map)}");
    if (floorRequestList.isNotEmpty) {
      for (var item in floorRequestList) {
        if (item.clientId == map['clientId']) {
          floorRequestModel = item;
        }
      }
      if (result == 1708) {
        floorRequestModel?.isRequestAccepted = false;
        floorRequestModel?.isRequestRejected = false;
        floorRequestModel?.isRequestReleased = true;
        floorRequestModel?.isMute = true;
        floorRequestModel?.isUnMute = false;
      } else if (result == 1709 ||
          result == 1712 ||
          result == 1711 ||
          result == 1707) {
        floorRequestList.remove(floorRequestModel);
      }
      showToastMessage(msg);
    }
  }

  @override
  void onInit() async{
    // TODO: implement onInit
    super.onInit();

    await GetStorage.init();
    if(showConfirmation) {
      EnxRtc.getPreview();
    } else{
      configureRoom();
    }
  }

  // Future<void> checkPermmision() async{
  //   bool isPermissionGranted =
  //   await handlePermissionsForCall(context);
  //   if (isPermissionGranted) {
  //     EnxRtc.getPreview();
  //   } else{
  //     Get.back();
  //
  //   }
  //
  //
  //
  // }

  @override
  void onReady() {
    // TODO: implement onReady
    super.onReady();
  }

  Future<void> configureRoom() async{
      initEnxRtc();
      _initForegroundTask();
      screenShare();

  }


  Future<void> rejectFloors(String clientId) async {
    EnxRtc.denyFloor(clientId);
  }

  Future<void> releaseFloors(String clientId) async {
    EnxRtc.releaseFloor(clientId);
  }

  Future<void> requestFloors() async {
    EnxRtc.requestFloor();
  }

  Future<void> screenShare() async {
    setting.addListener(() {
      if (Platform.isAndroid) {
        if (setting.isStart) {
          _startForegroundTask();
          EnxRtc.startScreenShare();
        } else {
          EnxRtc.stopScreenShare();
          _stopForegroundTask();
        }
      }
    });
  }

  Future scrollToBottom(ScrollController scrollController) async {
    while (scrollController.position.pixels !=
        scrollController.position.maxScrollExtent) {
      scrollController.jumpTo(scrollController.position.maxScrollExtent);
      await SchedulerBinding.instance!.endOfFrame;
    }
  }

  void sendMessage(
      String msg, bool isBroadCast, List<String> recipientIDs, String group) {
    EnxRtc.sendMessage(msg, isBroadCast, recipientIDs);
    // print("msf$msg id${recipientIDs.first}");

    chatModel = ChatModel(
        isBroadCast,
        recipientIDs.isNotEmpty ? recipientIDs.first : '',
        group,
        isBroadCast ? 'group' : 'private',
        msg,
        getCurrentTime(),
        false,
        FileShareModel(0, "", false),
        "",
        {});

    if (isBroadCast) {
      chatList.add(chatModel!);
    } else {
      print('rtg${privateChatListData.length}');

      privateChatList.add(chatModel!);
      if (getChatType == 'private') {
        for (var chatData in privateChatListData) {
          if (chatData.clientId == chatModel!.senderId) {
            chatData.chatList.add(chatModel!);
            print('rtg2${chatData.chatList.length}');
          }
        }
      }
    }

    scrollToBottom(scrollController);
    privateChatListData.refresh();

    update();
  }

  var recipientID;



  void showAlert(BuildContext context) {
    showDialog(
        context: context,
        builder: (context) => const AlertDialog(
              content: Text("hi"),
            ));
  }

  Future<void> startRecording() async {
    if (isRecord) {
      await EnxRtc.stopRecord();
    } else {
      await EnxRtc.startRecord();
    }
  }

  void switchAtView() {
    isGrid.value = !isGrid.value;
  }

  Future<void> switchCamera() async {
    var status = await Permission.camera.status;
    if (status.isDenied) {
      await Permission.camera.request();
    }

    backCamera.value = !backCamera.value;

    await EnxRtc.switchCamera();
  }

  Future<void> switchMedia(EnxController obj) async {
    deviceList.value = await EnxRtc.getDevices();
    print('dfc${deviceList}');
    // showSwitchAudioDialog(context,obj);

    String deviceName = '';
    print('dsvxc$selectedMedia');
    if (selectedMedia.contains('EARPIECE') ||
        selectedMedia.contains('Earpiece')) {
      if (Platform.isIOS) {
        deviceName = 'Speaker';
      } else {
        deviceName = 'SPEAKER_PHONE';
      }
    } else {
      if (Platform.isIOS) {
        deviceName = 'Earpiece';
      } else {
        deviceName = 'EARPIECE';
      }
    }
    print('dsvxc1$deviceName');

    await EnxRtc.switchMediaDevice(deviceName);
  }

  Future<void> toggleHardMuteAudio(String clientId) async {
    for (var model in participantsList) {
      if (model.clientId == clientId) {
        model.isAudioMuted = true;
      }
    }
    for (var model in moderatorsList) {
      if (model.clientId == clientId) {
        model.isAudioMuted = true;
      }
    }

    EnxRtc.hardMuteAudio(clientId);
  }

  Future<void> toggleHardMuteVideo(String clientId) async {
    for (var model in participantsList) {
      if (model.clientId == clientId) {
        model.isVideoMuted = true;
      }
    }
    for (var model in moderatorsList) {
      if (model.clientId == clientId) {
        model.isVideoMuted = true;
      }
    }
    EnxRtc.hardMuteVideo(clientId);

    update();
  }

  Future<void> toggleHardUnMuteAudio(String clientId) async {
    for (var model in participantsList) {
      if (model.clientId == clientId) {
        model.isAudioMuted = false;
      }
    }
    for (var model in moderatorsList) {
      if (model.clientId == clientId) {
        model.isAudioMuted = false;
      }
    }
    EnxRtc.hardUnMuteAudio(clientId);
  }

  // pending
  Future<void> toggleHardUnMuteVideo(String clientId) async {
    for (var model in participantsList) {
      if (model.clientId == clientId) {
        model.isVideoMuted = false;
      }
    }
    for (var model in moderatorsList) {
      if (model.clientId == clientId) {
        model.isVideoMuted = false;
      }
    }

    EnxRtc.hardUnMuteVideo(clientId);
  }

  Future<void> toggleMuteAudio() async {
    var status = await Permission.microphone.status;
    if (status.isDenied) {
      await Permission.microphone.request();
    }

    print("audioMute$isAudioMute");
    await EnxRtc.muteSelfAudio(!isAudioMute);
  }

  Future<void> toggleMuteVideo() async {
    var status = await Permission.camera.status;
    if (status.isDenied) {
      await Permission.camera.request();
    }
    //  isMuteVideo = !isMuteVideo;

    await EnxRtc.muteSelfVideo(!isVideoMute);
  }

  void updateMoreList(MoreModel model) {
    if (model.tag == MenuOption.recording) {
      startRecording();
    } else if (model.tag == MenuOption.switchAT) {
      switchAtView();
    } else if (model.tag == MenuOption.muteRoom) {
      muteRoom();
    }else if (model.tag == MenuOption.requestFloor){

      model.icon='assets/image/notification_icon_selected.png';

    }
/*
    else if (model.tag == MenuOption.annotationShare) {
      if( model.name=='Start Annotation'){
        isLongPress.value=true;
        showToastMessageLong('Please long press on player,On which you want to start Annotation');

      }else{
          annotationShare();


      }

    // annotationShare();
     // showAlert(context);
    }
*/
    update();
  }

  void _addEnxEventHandlers() {
    EnxRtc.onRoomConnected = (Map<dynamic, dynamic> map) {
      playAudio(newUser);

      print('onRoomConnectedFlutter' + jsonEncode(map));
      List streamjsonList = map['streams'] as List;

      print('streams ${streamjsonList.length}');
      progressValueInstream = ((streamjsonList.length + 1) / 100);
      roomID = map['id'];
      clientID = map["clientId"];
      setting.getSelfClientId(clientID);

      print('userList' + jsonEncode(map['userList']));
      print('user1List' + jsonEncode(map['user']));
      List jsonList = map['userList'] as List;
      appMode.value = map['room']['settings']['mode'];
      userModelSelf = UserModel(
          map['user']['clientId'],
          map['user']['name'],
          map['user']['role'],
          map['user']['audioMuted'],
          map['user']['videoMuted'],
          false);

      ParticipantsList activeTalkerListModel = ParticipantsList.fromJson(map);
      userList1.value = activeTalkerListModel.activeList!;
      userList.value = userList1.toSet().toList();

      addParticipantsList(userList);
      loadingTxt.value = 'Preparing ....';
      _dataAvailable.value = jsonEncode(map).isNotEmpty;
      print("list2${userList.length}");
      update();
      if (mode.toLowerCase() != 'Group'.toLowerCase()) {
        if (map['user']['role'].toLowerCase() == 'moderator') {
          EnxRtc.publish();
        }
      } else {
        EnxRtc.publish();
      }
      EnxRtc.getRole().then((value) => {
            if (value.toLowerCase() == 'moderator')
              {isModerator.value = true}
            else
              {isModerator.value = false}
          });
      moreListItem(map['user']['role']);
    };

    EnxRtc.onPublishedStream = (Map<dynamic, dynamic> map) {
      print('onPublishedStream' + jsonEncode(map));
      EnxRtc.setupVideo(0, 0, true, 300, 200);
      totalSubscriberInRoom.value =
          (totalSubscriberInRoom + progressValueInstream).toDouble();
      //    EnxRtc.switchMediaDevice("SPEAKER_PHONE");
      // _toggleSpeaker();
    };

    EnxRtc.onStreamAdded = (Map<dynamic, dynamic> map) {
      print('onStreamAdded' + jsonEncode(map));
      print('onStreamAdded Id' + map['streamId']);
      String streamId;

      streamId = map['streamId'];

      EnxRtc.subscribe(streamId);
    };
    EnxRtc.onSubscribedStream = (Map<dynamic, dynamic> map) {
      totalSubscriberInRoom.value =
          (totalSubscriberInRoom + progressValueInstream).toDouble();
    };

    EnxRtc.onRoomError = (Map<dynamic, dynamic> map) {
      print('onRoomError' + jsonEncode(map));
      connectError(map);
    };
    EnxRtc.onNotifyDeviceUpdate = (String deviceName) {
      print('onNotifyDeviceUpdate' + deviceName);
      if (deviceName.contains('EARPIECE') || deviceName.contains('Earpiece')) {
        if (Platform.isIOS) {
          selectedDevice.value = 'Earpiece';
        } else {
          selectedDevice.value = 'EARPIECE';
        }
      } else if (deviceName.contains('SPEAKER_PHONE') ||
          deviceName.contains('Speaker')) {
        if (Platform.isIOS) {
          selectedDevice.value = 'Speaker';
        } else {
          selectedDevice.value = 'SPEAKER_PHONE';
        }
      } else if (deviceName.contains('BLUETOOTH') ||
          deviceName.contains('Bluetooth')) {
        selectedDevice.value = 'BLUETOOTH';
      } else if (deviceName.contains('WIRED_HEADSET') ||
          deviceName.contains('Wired_headset') ||
          deviceName.contains('Headphones')) {
        if (Platform.isIOS) {
          selectedDevice.value = 'Headphones';
        } else {
          selectedDevice.value = 'WIRED_HEADSET';
        }
      }
    };

    EnxRtc.onActiveTalkerList = (Map<dynamic, dynamic> map) {
      print('onActiveTalkerList ' + map.toString());
      isActive.value = true;
      ActiveTalker activeTalkerListModel = ActiveTalker.fromJson(map);

      if (activeTalkerList.length == activeTalkerListModel.activeList!.length) {
        return;
      }

      activeTalkerList.value = activeTalkerListModel.activeList!;
      //   print('jsonListActiveTalker'+activeTalkerList[1].clientId.toString());
    };

    EnxRtc.onEventError = (Map<dynamic, dynamic> map) {
      print('onEventError' + jsonEncode(map));
      showToastMessage(map['msg']);
    };

    EnxRtc.onEventInfo = (Map<dynamic, dynamic> map) {
      print('onEventInfo' + jsonEncode(map));
    };

    EnxRtc.onUserConnected = (Map<dynamic, dynamic> map) {
      print('onUserConnected' + jsonEncode(map));
      settingList.map((element) => element.settingList.map((e){
        print('onUserConnectedSound' + e.name.toString());
        if(e.name.toString()==""){

        }
      }));
      UserListModel userListModel = UserListModel.fromJson(map);
      // userList1.add(userListModel);
      // userList.value=  userList1.toSet().toList();
      print('onEventInfo1 ${userList.length}');
      print(
          'list before ${moderatorsList.length} and ${participantsList.length}');
      var userModel = UserModel(map['clientId'], map['name'], map['role'],
          map['audioMuted'], map['videoMuted'], false);
      var chatModelList = ChatListModel(map['name'], map['clientId'], []);
      privateChatListData.add(chatModelList);

      if (userModel.role == 'participant') {
        if (participantsList.isEmpty) {
          participantsList.add(userModel);
        } else {
          for (var user in participantsList) {
            if (user.clientId != userModel.clientId) {
              participantsList.add(userModel);
            }
          }
        }
      } else {
        if (moderatorsList.isEmpty) {
          moderatorsList.add(userModel);
        } else {
          for (var user in moderatorsList) {
            if (user.clientId != userModel.clientId) {
              moderatorsList.add(userModel);
            }
          }
        }
      }

      print(
          'list after ${moderatorsList.length} and ${participantsList.length}');
      update();
      //  userList.add(map);
    };
    EnxRtc.onUserDisConnected = (Map<dynamic, dynamic> map) {
      print('onUserDisConnected' + jsonEncode(map));
      UserListModel userListModel = UserListModel.fromJson(map);

      participantsList.removeWhere((item) => item.clientId == map['clientId']);
      moderatorsList.removeWhere((item) => item.clientId == map['clientId']);
      privateChatListData
          .removeWhere((item) => item.clientId == map['clientId']);

      //update();
      print('onUserDisConnected2 ${userList.length}');
      //  disconnect(map);
    };

    EnxRtc.onRoomDisConnected = (Map<dynamic, dynamic> map) {
      playAudio(newUser);

      print('onRoomDisConnected' + jsonEncode(map));
      // Navigator.pop(context, '/Conference');
      if (map['msg'] == 'disconnected-by-moderator') {
        if (Get.currentRoute == "/ParticipantScreen") {
          print("dsa${Get.currentRoute}");
        }
        Get.back();
      }
      // if (Get.currentRoute == "/ChatPage") {
      //   print("dsa${Get.currentRoute}");
      //   Get.back();
      // }

      disconnect(map);
      isConfirmation.value=false;

    };

    EnxRtc.onAudioEvent = (Map<dynamic, dynamic> map) {
      print('onAudioEvent' + jsonEncode(map));

      if (map['msg'].toString() == "Audio Off") {
        isAudioMuted.value = true;
      } else {
        isAudioMuted.value = false;
      }

      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted =
              map['msg'].toString() == "Audio Off" ? true : false;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted =
              map['msg'].toString() == "Audio Off" ? true : false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };

    EnxRtc.onVideoEvent = (Map<dynamic, dynamic> map) {
      print('onVideoEvent' + jsonEncode(map));

      if (map['msg'].toString() == "Video Off") {
        isVideoMuted.value = true;
      } else {
        isVideoMuted.value = false;
      }

      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted =
              map['msg'].toString() == "Video Off" ? true : false;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted =
              map['msg'].toString() == "Video Off" ? true : false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onRoomRecordingOn = (Map<dynamic, dynamic> map) {
      print('onRoomRecordingOn' + jsonEncode(map));

      isRecording.value = true;
      for (var moreItem in moreList) {
        if (moreItem.tag == MenuOption.recording) {
          moreItem.status = true;
        }
      }
    };
    EnxRtc.onRoomRecordingOff = (Map<dynamic, dynamic> map) {
      print('onRoomRecordingOff' + jsonEncode(map));
      isRecording.value = false;
      for (var moreItem in moreList) {
        if (moreItem.tag == MenuOption.recording) {
          moreItem.status = false;
        }
      }
    };
    EnxRtc.onStartRecordingEvent = (Map<dynamic, dynamic> map) {
      print('onStartRecordingEvent' + jsonEncode(map));

      if (map['msg'].toString() == "Video Off") {
        isVideoMuted.value = true;
      } else {
        isVideoMuted.value = false;
      }
    };
    EnxRtc.onStopRecordingEvent = (Map<dynamic, dynamic> map) {
      print('onStopRecordingEvent' + jsonEncode(map));

      if (map['msg'].toString() == "Video Off") {
        isVideoMuted.value = true;
      } else {
        isVideoMuted.value = false;
      }
    };
    EnxRtc.onRemoteStreamAudioMute = (Map<dynamic, dynamic> map) {
      print('onRemoteStreamAudioMute' + jsonEncode(map));

      for (var model in participantsList) {
        if (model.clientId == map['clientId']) {
          model.isAudioMuted = true;
        }
      }

      for (var model in moderatorsList) {
        if (model.clientId == map['clientId']) {
          model.isAudioMuted = true;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();

      isHardMute.value = true;
    };
    EnxRtc.onRemoteStreamAudioUnMute = (Map<dynamic, dynamic> map) {
      print('onRemoteStreamAudioUnMute' + jsonEncode(map));
      for (var model in participantsList) {
        if (model.clientId == map['clientId']) {
          model.isAudioMuted = false;
        }
      }

      for (var model in moderatorsList) {
        if (model.clientId == map['clientId']) {
          model.isAudioMuted = false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();

      isHardMute.value = false;
    };
    EnxRtc.onRemoteStreamVideoMute = (Map<dynamic, dynamic> map) {
      for (var model in participantsList) {
        if (model.clientId == map['clientId']) {
          model.isVideoMuted = true;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == map['clientId']) {
          model.isVideoMuted = true;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onRemoteStreamVideoUnMute = (Map<dynamic, dynamic> map) {
      print('onReceivedHardMute' + jsonEncode(map));

      for (var model in participantsList) {
        if (model.clientId == map['clientId']) {
          model.isVideoMuted = false;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == map['clientId']) {
          model.isVideoMuted = false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onReceivedHardMute = (Map<dynamic, dynamic> map) {
      print('onReceivedHardMute' + jsonEncode(map));
      isHardMuteAudio.value = true;
    };
    EnxRtc.onReceivedHardUnMute = (Map<dynamic, dynamic> map) {
      print('onReceivedHardUnMute' + jsonEncode(map));
      isHardMuteAudio.value = false;
    };
    EnxRtc.onHardMuted = (Map<dynamic, dynamic> map) {
      print('onHardMuted' + jsonEncode(map));
    };
    EnxRtc.onHardUnMuted = (Map<dynamic, dynamic> map) {
      print('onHardUnMuted' + jsonEncode(map));
      update();
    };
    EnxRtc.onHardMutedAudio = (Map<dynamic, dynamic> map) {
      participantsList.refresh();
      moderatorsList.refresh();
      print('onHardMutedAudio' + jsonEncode(map));
    };
    EnxRtc.onHardUnMutedAudio = (Map<dynamic, dynamic> map) {
      print('onHardUnMutedAudio' + jsonEncode(map));
      participantsList.refresh();
      moderatorsList.refresh();
    };
    EnxRtc.onReceivedHardMuteAudio = (Map<dynamic, dynamic> map) {
      print('onReceivedHardMuteAudio' + jsonEncode(map));
      isHardMuteAudio.value = true;
      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted = true;
        }
      }

      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted = true;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onReceivedHardUnMuteAudio = (Map<dynamic, dynamic> map) {
      print('onReceivedHardUnMuteAudio' + jsonEncode(map));
      isHardMuteAudio.value = false;
      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted = false;
        }
      }

      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isAudioMuted = false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onHardMutedVideo = (Map<dynamic, dynamic> map) {
      print('onHardMutedVideo' + jsonEncode(map));
      participantsList.refresh();
      moderatorsList.refresh();
    };
    EnxRtc.onHardUnMutedVideo = (Map<dynamic, dynamic> map) {
      print('onHardUnMutedVideo' + jsonEncode(map));
      participantsList.refresh();
      moderatorsList.refresh();
    };
    EnxRtc.onReceivedHardMuteVideo = (Map<dynamic, dynamic> map) {
      print('onReceivedHardMuteVideo' + jsonEncode(map));
      hardMuteVideo.value = true;
      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted = true;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted = true;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onReceivedHardUnMuteVideo = (Map<dynamic, dynamic> map) {
      print('onReceivedHardUnMuteVideo' + jsonEncode(map));
      hardMuteVideo.value = false;
      for (var model in participantsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted = false;
        }
      }
      for (var model in moderatorsList) {
        if (model.clientId == userModelSelf!.clientId) {
          model.isVideoMuted = false;
        }
      }
      moderatorsList.refresh();
      participantsList.refresh();
    };
    EnxRtc.onOutBoundCallInitiated = (Map<dynamic, dynamic> map) {};
    EnxRtc.onUserAwaited = (Map<dynamic, dynamic> map) {
      var userModel = UserModel(map['clientId'], map['name'], map['role'],
          map['audioMuted'], map['videoMuted'], true);

      if (map['role'] == 'moderator') {
        moderatorsList.add(userModel);
      } else {
        participantsList.add(userModel);
      }
    };
    EnxRtc.onRoomAwaited = (Map<dynamic, dynamic> map) {};
    EnxRtc.onAcceptedFloorInvite = (Map<dynamic, dynamic> map) {};
    EnxRtc.onAckForApproveAwaitedUser == (Map<dynamic, dynamic> map) {};
    EnxRtc.onAckForDenyAwaitedUser = (Map<dynamic, dynamic> map) {};
    EnxRtc.onFloorRequested = (Map<dynamic, dynamic> map) {
      switch (map['result']) {
        case 1719:
          showToastMessage(map['msg']);
          break;
        case 1718:
          showToastMessage(map['msg']);
          break;
        case 1701:
        case 0:
        case 1702:
          participantFloorAction.value = 1;
          break;
      }
    };
    EnxRtc.onFloorRequestReceived = (Map<dynamic, dynamic> map) {
      floorRequestModel = FloorRequestModel(
          map['clientId'], map['name'], true, true, false, false, false);
      floorRequestList.add(floorRequestModel!);
      isNotificationIcon.value = true;
    };
    EnxRtc.onProcessFloorRequested = (Map<dynamic, dynamic> map) {
      print('onProcessFloorRequested${jsonEncode(map)}');
      final result = map['result'];
      final message = map['msg'];
      final request = map['request'];
      final params = map['request']['params'];
      final action = map['request']['params']['action'];
      print('1231$request ,$params,${map['request']['params']['clientId']}');
      if (floorRequestList.isNotEmpty) {
        for (var item in floorRequestList) {
          if (item.clientId == map['request']['params']['clientId']) {
            floorRequestModel = item;
          }
        }

        if (action == 'grantFloor' && result == 0) {
          floorRequestModel?.isRequestAccepted = false;
          floorRequestModel?.isRequestRejected = false;
          floorRequestModel?.isRequestReleased = true;
          floorRequestModel?.isMute = true;
          floorRequestModel?.isUnMute = false;
          isNotificationIcon.value = false;
        } else if (action == "denyFloor" && result == 0 ||
            action == "releaseFloor" && result == 0 ||
            result == 1711 ||
            result == 1707 ||
            result == 1709 ||
            result == 1712) {
          floorRequestList.remove(floorRequestModel);
          isNotificationIcon.value = false;
        }
      }
      showToastMessage("$action: $message");
    };
    EnxRtc.onGrantedFloorRequest = (Map<dynamic, dynamic> map) {
      if (!isRole) {
        participantFloorAction.value = 2;
      }
    };
    EnxRtc.onDeniedFloorRequest = (Map<dynamic, dynamic> map) {
      print('onDeniedFloorRequest${jsonEncode(map)}');
      participantFloorAction.value = 0;

      showToastMessage(map['msg']);
    };
    EnxRtc.onReleasedFloorRequest = (Map<dynamic, dynamic> map) {
      print('dfd${jsonEncode(map)}');
      if (isRole) {
        onGrantDenyReleaseManagement(map);
      } else {
        participantFloorAction.value = 0;

        showToastMessage(map['msg']);
      }
    };
    EnxRtc.onFloorCancelled = (Map<dynamic, dynamic> map) {
      print('onFloorCancelled${jsonEncode(map)}');
      participantFloorAction.value = 0;

      showToastMessage(map['msg']);
    };
    EnxRtc.onFloorFinished = (Map<dynamic, dynamic> map) {
      print('onFloorFinished${jsonEncode(map)}');
    };
    EnxRtc.onCancelledFloorRequest = (Map<dynamic, dynamic> map) {
      print('onCancelledFloorRequest${jsonEncode(map)}');
      for (var item in floorRequestList) {
        if (map['clientId'] == item.clientId) {
          floorRequestList.remove(item);
          update();
        }
      }
    };
    EnxRtc.onFinishedFloorRequest = (Map<dynamic, dynamic> map) {};
    EnxRtc.onACKInviteToFloorRequested = (Map<dynamic, dynamic> map) {};
    EnxRtc.onInviteToFloorRequested = (Map<dynamic, dynamic> map) {};
    EnxRtc.onInvitedForFloorAccess = (Map<dynamic, dynamic> map) {
      print('onCancelledFloorRequest${jsonEncode(map)}');
    };
    EnxRtc.onCanceledFloorInvite = (Map<dynamic, dynamic> map) {
      print('onCanceledFloorInvite${jsonEncode(map)}');
    };
    EnxRtc.onRejectedInviteFloor = (Map<dynamic, dynamic> map) {
      print('onRejectedInviteFloor${jsonEncode(map)}');
    };
    EnxRtc.onAcceptedFloorInvite = (Map<dynamic, dynamic> map) {};

    //chat listener
    EnxRtc.onMessageReceived = (Map<dynamic, dynamic> map) {
      print('onMessageReceived${jsonEncode(map)}');
      chatModel = ChatModel(
          map['broadcast'],
          map['senderId'],
          map['sender'],
          map['type'],
          map['message'],
          getCurrentTime(),
          true,
          FileShareModel(0, "", false),
          "",
          {});

      if (map['receipients'] != null) {
        //privateChatList.add(chatModel!);
        if (getChatType == 'private') {
          for (var chatData in privateChatListData) {
            if (chatData.clientId == map['senderId']) {
              chatData.chatList.add(chatModel!);
            }
          }
        }

        /*var privateChat=ChatListModel(map['sender'], map['senderId'], privateChatList);
    privateChatListData.add(privateChat);*/

      } else {
        if (Get.currentRoute != "/ChatPage") {
          isChatIcon.value = true;
        }
        chatList.add(chatModel!);
      }

      scrollToBottom(scrollController);
      privateChatListData.refresh();
      update();
    };
    EnxRtc.onUserDataReceived = (Map<dynamic, dynamic> map) {
      print('onUserDataReceived${jsonEncode(map)}');
       if(map['message']['type']=='message'){
         isTypingStarted.value=map['message']['typing'];
         typerName.value = map['sender'];
       }

    };
    EnxRtc.onAcknowledgedSendData = (Map<dynamic, dynamic> map) {
      print('onAcknowledgedSendData${jsonEncode(map)}');

    };
    EnxRtc.onScreenSharedStarted = (Map<dynamic, dynamic> map) {
      print('onScreenSharedStarted${jsonEncode(map)}');
      isScreenSharedStarted.value = true;
      screenShareStreamId.value = int.parse(map['streamId']);
      _portraitModeOnly();
    };
    EnxRtc.onScreenSharedStopped = (Map<dynamic, dynamic> map) {
      print('onScreenSharedStopped${jsonEncode(map)}');
      isScreenSharedStarted.value = false;
      _enableRotation();
    };
    EnxRtc.onStartScreenShareACK = (Map<dynamic, dynamic> map) {
      print('onStartScreenShareACK${jsonEncode(map)}');
      isScreenShared.value = true;

      setting.getScreenShare(true);
    };
    EnxRtc.onStoppedScreenShareACK = (Map<dynamic, dynamic> map) {
      print('onStoppedScreenShareACK${jsonEncode(map)}');
      isScreenShared.value = false;

      setting.getScreenShare(false);
    };
    EnxRtc.onExitScreenShareACK = (Map<dynamic, dynamic> map) {
      print('onExitScreenShareACK${jsonEncode(map)}');
    };

    EnxRtc.onAnnotationStarted = (Map<dynamic, dynamic> map) {
      print('onAnnotationStarted${jsonEncode(map)}');
      isScreenSharedStarted.value = true;
      screenShareStreamId.value = int.parse(map['streamId']);
      _portraitModeOnly();
    };
    EnxRtc.onAnnotationStopped = (Map<dynamic, dynamic> map) {
      print('onAnnotationStopped${jsonEncode(map)}');
      isScreenSharedStarted.value = false;
      _enableRotation();
    };
    EnxRtc.onStartAnnotationAck = (Map<dynamic, dynamic> map) {
      print('onStartAnnotationAck${jsonEncode(map)}');
      isScreenShared.value = true;
      isShowAnnotation.value = true;
      isLongPress.value = false;
      for (var moreItem in moreList) {
        if (moreItem.tag == MenuOption.annotationShare) {
          moreItem.name = 'Stop Annotation';
        }
      }
    };
    EnxRtc.onStoppedAnnotationAck = (Map<dynamic, dynamic> map) {
      print('onStoppedAnnotationAck${jsonEncode(map)}');
      isScreenShared.value = false;
      isLongPress.value = false;
      isShowAnnotation.value = false;

      for (var moreItem in moreList) {
        if (moreItem.tag == MenuOption.annotationShare) {
          moreItem.name = 'Start Annotation';
        }
      }
    };
    EnxRtc.onInitFileUpload = (Map<dynamic, dynamic> map) {
      // File upload started
      fileShareModel = FileShareModel(0, map['name'], false);
      print("onInitFileUpload " + map.toString());
      chatModel = ChatModel(
          getChatType == 'group' ? true : false,
          getChatType == 'group' ? "" : recipientID,
          getChatType,
          getChatType,
          "Your file: " + map['name'] + " is uploading",
          getCurrentTime(),
          false,
          fileShareModel,
          "",
          {});

      if (getChatType == 'group') {
        chatList.add(chatModel!);
      } else {
        print('rtg${privateChatListData.length}');

        privateChatList.add(chatModel!);
        if (getChatType == 'private') {
          for (var chatData in privateChatListData) {
            if (chatData.clientId == chatModel!.senderId) {
              chatData.chatList.add(chatModel!);
              print('rtg2${chatData.chatList.length}');
            }
          }
        }
      }
      scrollToBottom(scrollController);
      privateChatListData.refresh();

      update();
    };

    EnxRtc.onFileUploaded = (Map<dynamic, dynamic> map) {
      // File is uploaded
      print("onFileUploaded " + map.toString());
      if (getChatType == 'group') {
        for (var groupChatData in chatList) {
          if (groupChatData.fileShareModel!.fileName == map['name']) {
            groupChatData.message = "You shared a file File: " + map['name'];
          }
        }
      } else {
        for (var chatData in privateChatList) {
          if (chatData.fileShareModel!.fileName == map['name']) {
            chatData.message = "You shared a file File: " + map['name'];
          }
        }
      }
      privateChatListData.refresh();
      chatList.refresh();
      update();
    };

    EnxRtc.onFileUploadFailed = (Map<dynamic, dynamic> map) {
      // File upload failed
      print("onFileUploadFailed " + map.toString());
      if (getChatType == 'group') {
        for (var groupChatData in chatList) {
          if (groupChatData.fileShareModel!.fileName == map['name']) {
            groupChatData.message =
                "File Your file: " + map['name'] + "upload Failed";
          }
        }
      } else {
        for (var chatData in privateChatList) {
          if (chatData.fileShareModel!.fileName == map['name']) {
            chatData.message =
                "File Your file: " + map['name'] + "upload Failed";
          }
        }
      }

      update();
    };

    EnxRtc.onFileUploadStarted = (Map<dynamic, dynamic> map) {
      // File upload started... to recipient
      print("onFileUploadStarted " + map.toString());
    };

    EnxRtc.onFileAvailable = (Map<dynamic, dynamic> map) {
      // Files available for download... to recipient
      print("onFileAvailable " + map.toString());
      //{broadcast: true, index: 0, speed: 2114496.5357967666, expiresAt: 1664620367575, dlimit: 10, timeLimit: 86400, createdAt: 1664533967575, senderId: 5b828f8d-13e3-41b9-9658-7764ec136365, size: 915577KB, sender: Raj, name: Screenshot 2022-09-29 at 3.34.56 PM.png, time: 433}

      chatModel = ChatModel(
          map['broadcast'],
          map['senderId'],
          map['sender'],
          '',
          map['sender'] + " shared a file: " +  map['name'],

          getCurrentTime(),
          true,
          FileShareModel(0, "", false),
          map['name'],
          map);


      if( map['broadcast']){
        chatList.add(chatModel!);

      }else{

        privateChatList.add(chatModel!);
        if (getChatType == 'private') {
          for (var chatData in privateChatListData) {
            if (chatData.clientId == chatModel!.senderId) {
              chatData.chatList.add(chatModel!);
              print('rtg2${chatData.chatList.length}');
            }
          }
        }
      }
      scrollToBottom(scrollController);
      privateChatListData.refresh();

      update();

    };
    EnxRtc.onFileDownloaded = (Map<dynamic, dynamic> map) {
      // File has been downloaded
      print("onFileDownloaded " + map.toString());
      showToastMessage('File downloaded successfully at ${map['data']}');


    };



    EnxRtc.onFileDownloadFailed = (Map<dynamic, dynamic> map) {
      // File Download has failed
      print("onFileDownloadFailed " + map.toString());
      showToastMessage("Fail to download file.Please try again later");
    };
  }

  void _enableRotation() {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
      DeviceOrientation.landscapeLeft,
      DeviceOrientation.landscapeRight,
    ]);
  }

//For Android Screen share need foreground service
  Future<void> _initForegroundTask() async {
/*
    await FlutterForegroundTask.init(
      androidNotificationOptions: AndroidNotificationOptions(
        channelId: 'notification_channel_id',
        channelName: 'Foreground Notification',
        channelDescription:
            'This notification appears when the foreground service is running.',
        channelImportance: NotificationChannelImportance.LOW,
        priority: NotificationPriority.LOW,
        iconData: const NotificationIconData(
          resType: ResourceType.mipmap,
          resPrefix: ResourcePrefix.ic,
          name: 'launcher',
        ),
        buttons: [
          const NotificationButton(id: 'sendButton', text: 'Send'),
          const NotificationButton(id: 'testButton', text: 'Test'),
        ],
      ),
      printDevLog: true,
    );
*/
  }

  /// blocks rotation; sets orientation to: portrait
  void _portraitModeOnly() {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);
  }

  Future<bool> _startForegroundTask() async {
    ReceivePort? receivePort;
    bool reqResult;

   /* if (await FlutterForegroundTask.isRunningService) {
      receivePort = await FlutterForegroundTask.restartService();
    } else {
      receivePort = await FlutterForegroundTask.startService(
        notificationTitle: 'Foreground Service is running',
        notificationText: 'Tap to return to the app',
      );
    }
*/    if (receivePort != null) {
      _receivePort = receivePort;

      return true;
    }

    return false;
  }

  Future<bool> _stopForegroundTask() async {
    return await FlutterForegroundTask.stopService();
  }

  void _toggleSpeaker() async {
    List<dynamic> list = await EnxRtc.getDevices();

    print('deviceList:$list');

    EnxRtc.switchMediaDevice('SPEAKER_PHONE');
  }

  Future<void>  sendFile(bool isBroadcast, List<String> clientIds, String group) async{
    EnxRtc.sendFiles(isBroadcast, clientIds);
    recipientID = clientIds.isNotEmpty ? clientIds.first : '';
  }

  Future<void> downloadFile(Map<dynamic, dynamic> file) async{

    EnxRtc.downloadFile(file, true);

  }


  Future<void> typingIndicator(List<String> list, String type) async{

    Map<String, dynamic> jsonData = {
    'type': "message",
    'typing': isTyping.value,
    };

    if (type == 'group') {
      EnxRtc.sendUserData(jsonData, true, []);
    }else{
      EnxRtc.sendUserData(jsonData, false, list);
    }

    print('typing${jsonData}');

    
  }

}
